# -*- coding: utf-8 -*-

import logging


def set_log_level_for_dependency(debug):
    loggers = (
        logging.getLogger('flask'),
        logging.getLogger('werkzeug'),
    )

    handlers = {
        False: logging.NullHandler,
        True: logging.StreamHandler,
    }

    for logger in loggers:
        logger.handlers = []
        logger.handlers.append(handlers[bool(debug)]())
        logger.setLevel(logging.INFO if debug else logging.ERROR)


def endpoint(url_rule, method):
    return '{}({})'.format(method.upper(), url_rule)



# -*- coding: utf-8 -*-

import traceback

from . import runnable
from .utils.common import measure_time


class Script(runnable.RunnableObject):

    __run_point__ = None
    __stop_if_error__ = False
    __create_reason__ = False

    def __init__(self, program):
        super(Script, self).__init__()

        self._timer = None
        self.__program = program

    def __str__(self):
        return 'script ({}.{})'.format(
            self.__class__.__module__,
            self.__class__.__name__,
        )

    def __is_run__(self):
        return True

    @property
    def config(self):
        return self.__program.config

    @property
    def ext(self):
        return self.__program.ext

    def __call__(self, result, *args, **kwargs):
        if result.current_state.should_stop:
            return

        result.start(self)
        self._timer = measure_time()
        try:
            res = super(Script, self).__call__(result, *args, **kwargs)
            result.add_success(self, self._timer())
            return res
        except BaseException as error:
            if self.__stop_if_error__:
                result.current_state.should_stop = True

            runnable.set_debug_if_allowed(self.config)
            result.add_error(self, traceback.format_exc(), self._timer(), error)


class BeforeScript(Script):

    __run_point__ = 'before'


class AfterScript(Script):

    __run_point__ = 'after'



# -*- coding: utf-8 -*-

from functools import wraps
from contextlib import contextmanager

import requests

from . import constants
from .exceptions import MockerError
from ...utils.common import waiting_for
from .exceptions import MockServerClientError


instance = None


def handle_errors(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        try:
            return f(*args, **kwargs)
        except requests.ConnectionError:
            raise MockerError('Mock server was not started')
    return wrapper


class MockServerClient(object):

    ALLOW_HTTP_METHODS = (
        'get',
        'put',
        'head',
        'post',
        'patch',
        'delete',
        'options',
    )

    def __init__(self, config):
        """
        :type config: seismograph.ext.mocker.extension.Config
        """
        global instance
        instance = self

        self.__config = config

    def __getattr__(self, http_method):
        if http_method not in self.ALLOW_HTTP_METHODS:
            raise MockServerClientError(
                'Incorrect http request method: "{}"'.format(http_method),
            )

        method = getattr(requests, http_method)

        def method_wrapper(path, *args, **kwargs):
            return method(
                'http://{}:{}{}'.format(
                    self.__config.HOST,
                    self.__config.PORT,
                    path,
                ),
                *args, **kwargs
            )

        return method_wrapper

    @property
    def config(self):
        return self.__config

    @contextmanager
    @handle_errors
    def path(self, url_rule, **kwargs):
        self.add_mock(url_rule, **kwargs)
        try:
            yield
        finally:
            self.unblock_mock(url_rule, method=kwargs.get('method'))

    @handle_errors
    def add_mock(self,
                 url_rule,
                 body=None,
                 json=None,
                 html=None,
                 status=None,
                 method=None,
                 headers=None,
                 content_type=None):
        data = {'url_rule': url_rule}

        if method:
            data['method'] = method

        if headers:
            data['headers'] = headers

        if json:
            data['content_type'] = 'application/json'
        elif html:
            data['content_type'] = 'text/html'
        elif content_type:
            data['content_type'] = content_type

        if status:
            data['status'] = status

        data['body'] = body or json or html or ''

        def was_added(resp):
            d = resp.json()
            return d.get('status') == constants.MOCK_ADDED_STATUS

        get_resp = lambda: self.post('/mocker/api/mocks/add', json=data)

        return waiting_for(
            lambda: was_added(get_resp()),
            delay=0.5,
            timeout=self.__config.BLOCK_TIMEOUT,
            message='Can not add mock to url rule "{}" already exist'.format(url_rule),
        )

    @handle_errors
    def unblock_mock(self, url_rule, method=None):
        def was_unblocked(resp):
            d = resp.json()
            return d.get('status') == constants.OK_STATUS

        return was_unblocked(
            self.post(
                '/mocker/api/mocks/unblock',
                json={
                    'url_rule': url_rule,
                    'method': method or constants.DEFAULT_HTTP_METHOD,
                },
            ),
        )

    @handle_errors
    def clean_mocks(self):
        def was_cleaned(resp):
            d = resp.json()
            return d.get('status') == constants.OK_STATUS

        return was_cleaned(
            self.post(
                '/mocker/api/mocks/clean',
            )
        )



# -*- coding: utf-8 -*-

try:
    import httplib
except ImportError:
    import http.client as httplib

from flask import jsonify
from flask import request

from . import constants
from .mocks import BaseMock
from .tools import endpoint
from .mocks import get_mock_class_by_content_type


def make_response(status_code, *args, **kwargs):
    if args:
        resp = jsonify(results=args, count=len(args), **kwargs)
    else:
        resp = jsonify(kwargs)

    resp.status_code = status_code
    return resp


def make_error_response(status_code, *errors):
    resp = jsonify({'errors': errors})
    resp.status_code = status_code
    return resp


class MockerApi(object):

    def __init__(self, server):
        """
        :type server: seismograph.ext.mocker.server.MockServer
        """
        self.__server = server

    @property
    def url_rules(self):
        return {
            '/mocks/add': {
                'methods': ['POST'],
                'view_func': self.set_mock,
            },
            '/mocks/unblock': {
                'methods': ['POST'],
                'view_func': self.unblock_mock,
            },
            '/mocks/clean': {
                'methods': ['POST'],
                'view_func': self.clean_mocks,
            },
            '/status': {
                'methods': ['GET'],
                'view_func': self.get_status,
            },
        }

    def install(self):
        for url_rule, kwargs in self.url_rules.items():
            self.__server.add_url_rule(
                '{}{}'.format(constants.API_PATH, url_rule),
                **kwargs
            )

    def unblock_mock(self):
        data = request.json
        url_rule = data.get('url_rule')
        method = data.get('method', constants.DEFAULT_HTTP_METHOD)
        ep = endpoint(url_rule, method)

        if ep in self.__server.views:
            del self.__server.views[ep]

        return make_response(httplib.OK, status=constants.OK_STATUS)

    def set_mock(self):
        data = request.json
        url_rule = data.get('url_rule')

        if not url_rule:
            return make_error_response(httplib.BAD_REQUEST, '"url_rule" is required param')

        content_type = data.get('content_type')
        method = data.get('method', constants.DEFAULT_HTTP_METHOD)
        status = data.get('status', httplib.OK)
        headers = data.get('headers', {})
        body = data.get('body', '')

        if endpoint(url_rule, method) in self.__server.views:
            return make_response(httplib.OK, status=constants.MOCK_BLOCKED_STATUS)

        mock_cls = get_mock_class_by_content_type(content_type)

        self.__server.add_mock(
            mock_cls(
                url_rule,
                body=body,
                headers=headers,
                status_code=status,
                http_method=method,
                mime_type=content_type,
                content_type=content_type,
            ),
        )

        return make_response(httplib.OK, status=constants.MOCK_ADDED_STATUS)

    def clean_mocks(self):
        for ep, view_func in self.__server.views.items():
            if isinstance(view_func, BaseMock):
                del self.__server.views[ep]

        return make_response(httplib.OK, status=constants.OK_STATUS)

    def get_status(self):
        data = {
            'host': self.__server.config.HOST,
            'port': self.__server.config.PORT,
            'debug': self.__server.config.DEBUG,
            'path_to_mocks': self.__server.config.PATH_TO_MOCKS,
            'block_timeout': self.__server.config.BLOCK_TIMEOUT,
            'urls': [ep for ep, mck in self.__server.views.items() if isinstance(mck, BaseMock)],
        }

        return make_response(httplib.OK, **data)



# -*- coding: utf-8 -*-

from ...exceptions import SeismographError


class MockerError(SeismographError):
    pass


class MockServerError(MockerError):
    pass


class MockServerClientError(MockerError):
    pass
